// Generated by gir (https://github.com/gtk-rs/gir @ 0e476ab5c1de)
// from 
// from gir-files (https://github.com/gtk-rs/gir-files.git @ cfc0305f903b)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "AdwAboutDialogClass", sizeof(AdwAboutDialogClass), alignof(AdwAboutDialogClass));
    printf("%s;%zu;%zu\n", "AdwAboutWindowClass", sizeof(AdwAboutWindowClass), alignof(AdwAboutWindowClass));
    printf("%s;%zu;%zu\n", "AdwActionRow", sizeof(AdwActionRow), alignof(AdwActionRow));
    printf("%s;%zu;%zu\n", "AdwActionRowClass", sizeof(AdwActionRowClass), alignof(AdwActionRowClass));
    printf("%s;%zu;%zu\n", "AdwAlertDialog", sizeof(AdwAlertDialog), alignof(AdwAlertDialog));
    printf("%s;%zu;%zu\n", "AdwAlertDialogClass", sizeof(AdwAlertDialogClass), alignof(AdwAlertDialogClass));
    printf("%s;%zu;%zu\n", "AdwAnimation", sizeof(AdwAnimation), alignof(AdwAnimation));
    printf("%s;%zu;%zu\n", "AdwAnimationState", sizeof(AdwAnimationState), alignof(AdwAnimationState));
    printf("%s;%zu;%zu\n", "AdwApplication", sizeof(AdwApplication), alignof(AdwApplication));
    printf("%s;%zu;%zu\n", "AdwApplicationClass", sizeof(AdwApplicationClass), alignof(AdwApplicationClass));
    printf("%s;%zu;%zu\n", "AdwApplicationWindow", sizeof(AdwApplicationWindow), alignof(AdwApplicationWindow));
    printf("%s;%zu;%zu\n", "AdwApplicationWindowClass", sizeof(AdwApplicationWindowClass), alignof(AdwApplicationWindowClass));
    printf("%s;%zu;%zu\n", "AdwAvatarClass", sizeof(AdwAvatarClass), alignof(AdwAvatarClass));
    printf("%s;%zu;%zu\n", "AdwBannerClass", sizeof(AdwBannerClass), alignof(AdwBannerClass));
    printf("%s;%zu;%zu\n", "AdwBin", sizeof(AdwBin), alignof(AdwBin));
    printf("%s;%zu;%zu\n", "AdwBinClass", sizeof(AdwBinClass), alignof(AdwBinClass));
    printf("%s;%zu;%zu\n", "AdwBreakpointBin", sizeof(AdwBreakpointBin), alignof(AdwBreakpointBin));
    printf("%s;%zu;%zu\n", "AdwBreakpointBinClass", sizeof(AdwBreakpointBinClass), alignof(AdwBreakpointBinClass));
    printf("%s;%zu;%zu\n", "AdwBreakpointClass", sizeof(AdwBreakpointClass), alignof(AdwBreakpointClass));
    printf("%s;%zu;%zu\n", "AdwBreakpointConditionLengthType", sizeof(AdwBreakpointConditionLengthType), alignof(AdwBreakpointConditionLengthType));
    printf("%s;%zu;%zu\n", "AdwBreakpointConditionRatioType", sizeof(AdwBreakpointConditionRatioType), alignof(AdwBreakpointConditionRatioType));
    printf("%s;%zu;%zu\n", "AdwButtonContentClass", sizeof(AdwButtonContentClass), alignof(AdwButtonContentClass));
    printf("%s;%zu;%zu\n", "AdwCarouselClass", sizeof(AdwCarouselClass), alignof(AdwCarouselClass));
    printf("%s;%zu;%zu\n", "AdwCarouselIndicatorDotsClass", sizeof(AdwCarouselIndicatorDotsClass), alignof(AdwCarouselIndicatorDotsClass));
    printf("%s;%zu;%zu\n", "AdwCarouselIndicatorLinesClass", sizeof(AdwCarouselIndicatorLinesClass), alignof(AdwCarouselIndicatorLinesClass));
    printf("%s;%zu;%zu\n", "AdwCenteringPolicy", sizeof(AdwCenteringPolicy), alignof(AdwCenteringPolicy));
    printf("%s;%zu;%zu\n", "AdwClampClass", sizeof(AdwClampClass), alignof(AdwClampClass));
    printf("%s;%zu;%zu\n", "AdwClampLayoutClass", sizeof(AdwClampLayoutClass), alignof(AdwClampLayoutClass));
    printf("%s;%zu;%zu\n", "AdwClampScrollableClass", sizeof(AdwClampScrollableClass), alignof(AdwClampScrollableClass));
    printf("%s;%zu;%zu\n", "AdwColorScheme", sizeof(AdwColorScheme), alignof(AdwColorScheme));
    printf("%s;%zu;%zu\n", "AdwComboRow", sizeof(AdwComboRow), alignof(AdwComboRow));
    printf("%s;%zu;%zu\n", "AdwComboRowClass", sizeof(AdwComboRowClass), alignof(AdwComboRowClass));
    printf("%s;%zu;%zu\n", "AdwDialog", sizeof(AdwDialog), alignof(AdwDialog));
    printf("%s;%zu;%zu\n", "AdwDialogClass", sizeof(AdwDialogClass), alignof(AdwDialogClass));
    printf("%s;%zu;%zu\n", "AdwDialogPresentationMode", sizeof(AdwDialogPresentationMode), alignof(AdwDialogPresentationMode));
    printf("%s;%zu;%zu\n", "AdwEasing", sizeof(AdwEasing), alignof(AdwEasing));
    printf("%s;%zu;%zu\n", "AdwEntryRow", sizeof(AdwEntryRow), alignof(AdwEntryRow));
    printf("%s;%zu;%zu\n", "AdwEntryRowClass", sizeof(AdwEntryRowClass), alignof(AdwEntryRowClass));
    printf("%s;%zu;%zu\n", "AdwEnumListItemClass", sizeof(AdwEnumListItemClass), alignof(AdwEnumListItemClass));
    printf("%s;%zu;%zu\n", "AdwEnumListModelClass", sizeof(AdwEnumListModelClass), alignof(AdwEnumListModelClass));
    printf("%s;%zu;%zu\n", "AdwExpanderRow", sizeof(AdwExpanderRow), alignof(AdwExpanderRow));
    printf("%s;%zu;%zu\n", "AdwExpanderRowClass", sizeof(AdwExpanderRowClass), alignof(AdwExpanderRowClass));
    printf("%s;%zu;%zu\n", "AdwFlapClass", sizeof(AdwFlapClass), alignof(AdwFlapClass));
    printf("%s;%zu;%zu\n", "AdwFlapFoldPolicy", sizeof(AdwFlapFoldPolicy), alignof(AdwFlapFoldPolicy));
    printf("%s;%zu;%zu\n", "AdwFlapTransitionType", sizeof(AdwFlapTransitionType), alignof(AdwFlapTransitionType));
    printf("%s;%zu;%zu\n", "AdwFoldThresholdPolicy", sizeof(AdwFoldThresholdPolicy), alignof(AdwFoldThresholdPolicy));
    printf("%s;%zu;%zu\n", "AdwHeaderBarClass", sizeof(AdwHeaderBarClass), alignof(AdwHeaderBarClass));
    printf("%s;%zu;%zu\n", "AdwLeafletClass", sizeof(AdwLeafletClass), alignof(AdwLeafletClass));
    printf("%s;%zu;%zu\n", "AdwLeafletPageClass", sizeof(AdwLeafletPageClass), alignof(AdwLeafletPageClass));
    printf("%s;%zu;%zu\n", "AdwLeafletTransitionType", sizeof(AdwLeafletTransitionType), alignof(AdwLeafletTransitionType));
    printf("%s;%zu;%zu\n", "AdwLengthUnit", sizeof(AdwLengthUnit), alignof(AdwLengthUnit));
    printf("%s;%zu;%zu\n", "AdwMessageDialog", sizeof(AdwMessageDialog), alignof(AdwMessageDialog));
    printf("%s;%zu;%zu\n", "AdwMessageDialogClass", sizeof(AdwMessageDialogClass), alignof(AdwMessageDialogClass));
    printf("%s;%zu;%zu\n", "AdwNavigationDirection", sizeof(AdwNavigationDirection), alignof(AdwNavigationDirection));
    printf("%s;%zu;%zu\n", "AdwNavigationPage", sizeof(AdwNavigationPage), alignof(AdwNavigationPage));
    printf("%s;%zu;%zu\n", "AdwNavigationPageClass", sizeof(AdwNavigationPageClass), alignof(AdwNavigationPageClass));
    printf("%s;%zu;%zu\n", "AdwNavigationSplitViewClass", sizeof(AdwNavigationSplitViewClass), alignof(AdwNavigationSplitViewClass));
    printf("%s;%zu;%zu\n", "AdwNavigationViewClass", sizeof(AdwNavigationViewClass), alignof(AdwNavigationViewClass));
    printf("%s;%zu;%zu\n", "AdwOverlaySplitViewClass", sizeof(AdwOverlaySplitViewClass), alignof(AdwOverlaySplitViewClass));
    printf("%s;%zu;%zu\n", "AdwPasswordEntryRowClass", sizeof(AdwPasswordEntryRowClass), alignof(AdwPasswordEntryRowClass));
    printf("%s;%zu;%zu\n", "AdwPreferencesDialog", sizeof(AdwPreferencesDialog), alignof(AdwPreferencesDialog));
    printf("%s;%zu;%zu\n", "AdwPreferencesDialogClass", sizeof(AdwPreferencesDialogClass), alignof(AdwPreferencesDialogClass));
    printf("%s;%zu;%zu\n", "AdwPreferencesGroup", sizeof(AdwPreferencesGroup), alignof(AdwPreferencesGroup));
    printf("%s;%zu;%zu\n", "AdwPreferencesGroupClass", sizeof(AdwPreferencesGroupClass), alignof(AdwPreferencesGroupClass));
    printf("%s;%zu;%zu\n", "AdwPreferencesPage", sizeof(AdwPreferencesPage), alignof(AdwPreferencesPage));
    printf("%s;%zu;%zu\n", "AdwPreferencesPageClass", sizeof(AdwPreferencesPageClass), alignof(AdwPreferencesPageClass));
    printf("%s;%zu;%zu\n", "AdwPreferencesRow", sizeof(AdwPreferencesRow), alignof(AdwPreferencesRow));
    printf("%s;%zu;%zu\n", "AdwPreferencesRowClass", sizeof(AdwPreferencesRowClass), alignof(AdwPreferencesRowClass));
    printf("%s;%zu;%zu\n", "AdwPreferencesWindow", sizeof(AdwPreferencesWindow), alignof(AdwPreferencesWindow));
    printf("%s;%zu;%zu\n", "AdwPreferencesWindowClass", sizeof(AdwPreferencesWindowClass), alignof(AdwPreferencesWindowClass));
    printf("%s;%zu;%zu\n", "AdwResponseAppearance", sizeof(AdwResponseAppearance), alignof(AdwResponseAppearance));
    printf("%s;%zu;%zu\n", "AdwSpinRowClass", sizeof(AdwSpinRowClass), alignof(AdwSpinRowClass));
    printf("%s;%zu;%zu\n", "AdwSplitButtonClass", sizeof(AdwSplitButtonClass), alignof(AdwSplitButtonClass));
    printf("%s;%zu;%zu\n", "AdwSqueezerClass", sizeof(AdwSqueezerClass), alignof(AdwSqueezerClass));
    printf("%s;%zu;%zu\n", "AdwSqueezerPageClass", sizeof(AdwSqueezerPageClass), alignof(AdwSqueezerPageClass));
    printf("%s;%zu;%zu\n", "AdwSqueezerTransitionType", sizeof(AdwSqueezerTransitionType), alignof(AdwSqueezerTransitionType));
    printf("%s;%zu;%zu\n", "AdwStatusPageClass", sizeof(AdwStatusPageClass), alignof(AdwStatusPageClass));
    printf("%s;%zu;%zu\n", "AdwStyleManagerClass", sizeof(AdwStyleManagerClass), alignof(AdwStyleManagerClass));
    printf("%s;%zu;%zu\n", "AdwSwipeTrackerClass", sizeof(AdwSwipeTrackerClass), alignof(AdwSwipeTrackerClass));
    printf("%s;%zu;%zu\n", "AdwSwipeableInterface", sizeof(AdwSwipeableInterface), alignof(AdwSwipeableInterface));
    printf("%s;%zu;%zu\n", "AdwSwitchRowClass", sizeof(AdwSwitchRowClass), alignof(AdwSwitchRowClass));
    printf("%s;%zu;%zu\n", "AdwTabBarClass", sizeof(AdwTabBarClass), alignof(AdwTabBarClass));
    printf("%s;%zu;%zu\n", "AdwTabButtonClass", sizeof(AdwTabButtonClass), alignof(AdwTabButtonClass));
    printf("%s;%zu;%zu\n", "AdwTabOverviewClass", sizeof(AdwTabOverviewClass), alignof(AdwTabOverviewClass));
    printf("%s;%zu;%zu\n", "AdwTabPageClass", sizeof(AdwTabPageClass), alignof(AdwTabPageClass));
    printf("%s;%zu;%zu\n", "AdwTabViewClass", sizeof(AdwTabViewClass), alignof(AdwTabViewClass));
    printf("%s;%zu;%zu\n", "AdwTabViewShortcuts", sizeof(AdwTabViewShortcuts), alignof(AdwTabViewShortcuts));
    printf("%s;%zu;%zu\n", "AdwToastClass", sizeof(AdwToastClass), alignof(AdwToastClass));
    printf("%s;%zu;%zu\n", "AdwToastOverlayClass", sizeof(AdwToastOverlayClass), alignof(AdwToastOverlayClass));
    printf("%s;%zu;%zu\n", "AdwToastPriority", sizeof(AdwToastPriority), alignof(AdwToastPriority));
    printf("%s;%zu;%zu\n", "AdwToolbarStyle", sizeof(AdwToolbarStyle), alignof(AdwToolbarStyle));
    printf("%s;%zu;%zu\n", "AdwToolbarViewClass", sizeof(AdwToolbarViewClass), alignof(AdwToolbarViewClass));
    printf("%s;%zu;%zu\n", "AdwViewStackClass", sizeof(AdwViewStackClass), alignof(AdwViewStackClass));
    printf("%s;%zu;%zu\n", "AdwViewStackPageClass", sizeof(AdwViewStackPageClass), alignof(AdwViewStackPageClass));
    printf("%s;%zu;%zu\n", "AdwViewStackPagesClass", sizeof(AdwViewStackPagesClass), alignof(AdwViewStackPagesClass));
    printf("%s;%zu;%zu\n", "AdwViewSwitcherBarClass", sizeof(AdwViewSwitcherBarClass), alignof(AdwViewSwitcherBarClass));
    printf("%s;%zu;%zu\n", "AdwViewSwitcherClass", sizeof(AdwViewSwitcherClass), alignof(AdwViewSwitcherClass));
    printf("%s;%zu;%zu\n", "AdwViewSwitcherPolicy", sizeof(AdwViewSwitcherPolicy), alignof(AdwViewSwitcherPolicy));
    printf("%s;%zu;%zu\n", "AdwViewSwitcherTitleClass", sizeof(AdwViewSwitcherTitleClass), alignof(AdwViewSwitcherTitleClass));
    printf("%s;%zu;%zu\n", "AdwWindow", sizeof(AdwWindow), alignof(AdwWindow));
    printf("%s;%zu;%zu\n", "AdwWindowClass", sizeof(AdwWindowClass), alignof(AdwWindowClass));
    printf("%s;%zu;%zu\n", "AdwWindowTitleClass", sizeof(AdwWindowTitleClass), alignof(AdwWindowTitleClass));
    return 0;
}
