// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GArray", sizeof(GArray), alignof(GArray));
    printf("%s;%zu;%zu\n", "GAsciiType", sizeof(GAsciiType), alignof(GAsciiType));
    printf("%s;%zu;%zu\n", "GBookmarkFileError", sizeof(GBookmarkFileError), alignof(GBookmarkFileError));
    printf("%s;%zu;%zu\n", "GByteArray", sizeof(GByteArray), alignof(GByteArray));
    printf("%s;%zu;%zu\n", "GChecksumType", sizeof(GChecksumType), alignof(GChecksumType));
    printf("%s;%zu;%zu\n", "GCompletion", sizeof(GCompletion), alignof(GCompletion));
    printf("%s;%zu;%zu\n", "GCond", sizeof(GCond), alignof(GCond));
    printf("%s;%zu;%zu\n", "GConvertError", sizeof(GConvertError), alignof(GConvertError));
    printf("%s;%zu;%zu\n", "GDate", sizeof(GDate), alignof(GDate));
    printf("%s;%zu;%zu\n", "GDateDMY", sizeof(GDateDMY), alignof(GDateDMY));
    printf("%s;%zu;%zu\n", "GDateDay", sizeof(GDateDay), alignof(GDateDay));
    printf("%s;%zu;%zu\n", "GDateMonth", sizeof(GDateMonth), alignof(GDateMonth));
    printf("%s;%zu;%zu\n", "GDateWeekday", sizeof(GDateWeekday), alignof(GDateWeekday));
    printf("%s;%zu;%zu\n", "GDateYear", sizeof(GDateYear), alignof(GDateYear));
    printf("%s;%zu;%zu\n", "GDebugKey", sizeof(GDebugKey), alignof(GDebugKey));
    printf("%s;%zu;%zu\n", "GError", sizeof(GError), alignof(GError));
    printf("%s;%zu;%zu\n", "GErrorType", sizeof(GErrorType), alignof(GErrorType));
    printf("%s;%zu;%zu\n", "GFileError", sizeof(GFileError), alignof(GFileError));
    printf("%s;%zu;%zu\n", "GFileSetContentsFlags", sizeof(GFileSetContentsFlags), alignof(GFileSetContentsFlags));
    printf("%s;%zu;%zu\n", "GFileTest", sizeof(GFileTest), alignof(GFileTest));
    printf("%s;%zu;%zu\n", "GFormatSizeFlags", sizeof(GFormatSizeFlags), alignof(GFormatSizeFlags));
    printf("%s;%zu;%zu\n", "GHashTableIter", sizeof(GHashTableIter), alignof(GHashTableIter));
    printf("%s;%zu;%zu\n", "GHook", sizeof(GHook), alignof(GHook));
    printf("%s;%zu;%zu\n", "GHookFlagMask", sizeof(GHookFlagMask), alignof(GHookFlagMask));
    printf("%s;%zu;%zu\n", "GHookList", sizeof(GHookList), alignof(GHookList));
    printf("%s;%zu;%zu\n", "GIOChannelError", sizeof(GIOChannelError), alignof(GIOChannelError));
    printf("%s;%zu;%zu\n", "GIOCondition", sizeof(GIOCondition), alignof(GIOCondition));
    printf("%s;%zu;%zu\n", "GIOError", sizeof(GIOError), alignof(GIOError));
    printf("%s;%zu;%zu\n", "GIOFlags", sizeof(GIOFlags), alignof(GIOFlags));
    printf("%s;%zu;%zu\n", "GIOFuncs", sizeof(GIOFuncs), alignof(GIOFuncs));
    printf("%s;%zu;%zu\n", "GIOStatus", sizeof(GIOStatus), alignof(GIOStatus));
    printf("%s;%zu;%zu\n", "GKeyFileError", sizeof(GKeyFileError), alignof(GKeyFileError));
    printf("%s;%zu;%zu\n", "GKeyFileFlags", sizeof(GKeyFileFlags), alignof(GKeyFileFlags));
    printf("%s;%zu;%zu\n", "GList", sizeof(GList), alignof(GList));
    printf("%s;%zu;%zu\n", "GLogField", sizeof(GLogField), alignof(GLogField));
    printf("%s;%zu;%zu\n", "GLogLevelFlags", sizeof(GLogLevelFlags), alignof(GLogLevelFlags));
    printf("%s;%zu;%zu\n", "GLogWriterOutput", sizeof(GLogWriterOutput), alignof(GLogWriterOutput));
    printf("%s;%zu;%zu\n", "GMainContextFlags", sizeof(GMainContextFlags), alignof(GMainContextFlags));
    printf("%s;%zu;%zu\n", "GMarkupCollectType", sizeof(GMarkupCollectType), alignof(GMarkupCollectType));
    printf("%s;%zu;%zu\n", "GMarkupError", sizeof(GMarkupError), alignof(GMarkupError));
    printf("%s;%zu;%zu\n", "GMarkupParseFlags", sizeof(GMarkupParseFlags), alignof(GMarkupParseFlags));
    printf("%s;%zu;%zu\n", "GMarkupParser", sizeof(GMarkupParser), alignof(GMarkupParser));
    printf("%s;%zu;%zu\n", "GMemVTable", sizeof(GMemVTable), alignof(GMemVTable));
    printf("%s;%zu;%zu\n", "GMutex", sizeof(GMutex), alignof(GMutex));
    printf("%s;%zu;%zu\n", "GNode", sizeof(GNode), alignof(GNode));
    printf("%s;%zu;%zu\n", "GNormalizeMode", sizeof(GNormalizeMode), alignof(GNormalizeMode));
    printf("%s;%zu;%zu\n", "GNumberParserError", sizeof(GNumberParserError), alignof(GNumberParserError));
    printf("%s;%zu;%zu\n", "GOnce", sizeof(GOnce), alignof(GOnce));
    printf("%s;%zu;%zu\n", "GOnceStatus", sizeof(GOnceStatus), alignof(GOnceStatus));
    printf("%s;%zu;%zu\n", "GOptionArg", sizeof(GOptionArg), alignof(GOptionArg));
    printf("%s;%zu;%zu\n", "GOptionEntry", sizeof(GOptionEntry), alignof(GOptionEntry));
    printf("%s;%zu;%zu\n", "GOptionError", sizeof(GOptionError), alignof(GOptionError));
    printf("%s;%zu;%zu\n", "GOptionFlags", sizeof(GOptionFlags), alignof(GOptionFlags));
    printf("%s;%zu;%zu\n", "GPathBuf", sizeof(GPathBuf), alignof(GPathBuf));
    printf("%s;%zu;%zu\n", "GPid", sizeof(GPid), alignof(GPid));
    printf("%s;%zu;%zu\n", "GPollFD", sizeof(GPollFD), alignof(GPollFD));
    printf("%s;%zu;%zu\n", "GPrivate", sizeof(GPrivate), alignof(GPrivate));
    printf("%s;%zu;%zu\n", "GPtrArray", sizeof(GPtrArray), alignof(GPtrArray));
    printf("%s;%zu;%zu\n", "GQuark", sizeof(GQuark), alignof(GQuark));
    printf("%s;%zu;%zu\n", "GQueue", sizeof(GQueue), alignof(GQueue));
    printf("%s;%zu;%zu\n", "GRWLock", sizeof(GRWLock), alignof(GRWLock));
    printf("%s;%zu;%zu\n", "GRecMutex", sizeof(GRecMutex), alignof(GRecMutex));
    printf("%s;%zu;%zu\n", "GRefString", sizeof(GRefString), alignof(GRefString));
    printf("%s;%zu;%zu\n", "GRegexCompileFlags", sizeof(GRegexCompileFlags), alignof(GRegexCompileFlags));
    printf("%s;%zu;%zu\n", "GRegexError", sizeof(GRegexError), alignof(GRegexError));
    printf("%s;%zu;%zu\n", "GRegexMatchFlags", sizeof(GRegexMatchFlags), alignof(GRegexMatchFlags));
    printf("%s;%zu;%zu\n", "GSList", sizeof(GSList), alignof(GSList));
    printf("%s;%zu;%zu\n", "GScanner", sizeof(GScanner), alignof(GScanner));
    printf("%s;%zu;%zu\n", "GSeekType", sizeof(GSeekType), alignof(GSeekType));
    printf("%s;%zu;%zu\n", "GShellError", sizeof(GShellError), alignof(GShellError));
    printf("%s;%zu;%zu\n", "GSliceConfig", sizeof(GSliceConfig), alignof(GSliceConfig));
    printf("%s;%zu;%zu\n", "GSource", sizeof(GSource), alignof(GSource));
    printf("%s;%zu;%zu\n", "GSourceCallbackFuncs", sizeof(GSourceCallbackFuncs), alignof(GSourceCallbackFuncs));
    printf("%s;%zu;%zu\n", "GSourceFuncs", sizeof(GSourceFuncs), alignof(GSourceFuncs));
    printf("%s;%zu;%zu\n", "GSpawnError", sizeof(GSpawnError), alignof(GSpawnError));
    printf("%s;%zu;%zu\n", "GSpawnFlags", sizeof(GSpawnFlags), alignof(GSpawnFlags));
    printf("%s;%zu;%zu\n", "GStaticMutex", sizeof(GStaticMutex), alignof(GStaticMutex));
    printf("%s;%zu;%zu\n", "GStaticPrivate", sizeof(GStaticPrivate), alignof(GStaticPrivate));
    printf("%s;%zu;%zu\n", "GStaticRWLock", sizeof(GStaticRWLock), alignof(GStaticRWLock));
    printf("%s;%zu;%zu\n", "GStaticRecMutex", sizeof(GStaticRecMutex), alignof(GStaticRecMutex));
    printf("%s;%zu;%zu\n", "GString", sizeof(GString), alignof(GString));
    printf("%s;%zu;%zu\n", "GStrv", sizeof(GStrv), alignof(GStrv));
    printf("%s;%zu;%zu\n", "GTestConfig", sizeof(GTestConfig), alignof(GTestConfig));
    printf("%s;%zu;%zu\n", "GTestFileType", sizeof(GTestFileType), alignof(GTestFileType));
    printf("%s;%zu;%zu\n", "GTestLogBuffer", sizeof(GTestLogBuffer), alignof(GTestLogBuffer));
    printf("%s;%zu;%zu\n", "GTestLogType", sizeof(GTestLogType), alignof(GTestLogType));
    printf("%s;%zu;%zu\n", "GTestResult", sizeof(GTestResult), alignof(GTestResult));
    printf("%s;%zu;%zu\n", "GTestSubprocessFlags", sizeof(GTestSubprocessFlags), alignof(GTestSubprocessFlags));
    printf("%s;%zu;%zu\n", "GTestTrapFlags", sizeof(GTestTrapFlags), alignof(GTestTrapFlags));
    printf("%s;%zu;%zu\n", "GThread", sizeof(GThread), alignof(GThread));
    printf("%s;%zu;%zu\n", "GThreadError", sizeof(GThreadError), alignof(GThreadError));
    printf("%s;%zu;%zu\n", "GThreadFunctions", sizeof(GThreadFunctions), alignof(GThreadFunctions));
    printf("%s;%zu;%zu\n", "GThreadPool", sizeof(GThreadPool), alignof(GThreadPool));
    printf("%s;%zu;%zu\n", "GThreadPriority", sizeof(GThreadPriority), alignof(GThreadPriority));
    printf("%s;%zu;%zu\n", "GTime", sizeof(GTime), alignof(GTime));
    printf("%s;%zu;%zu\n", "GTimeSpan", sizeof(GTimeSpan), alignof(GTimeSpan));
    printf("%s;%zu;%zu\n", "GTimeType", sizeof(GTimeType), alignof(GTimeType));
    printf("%s;%zu;%zu\n", "GTimeVal", sizeof(GTimeVal), alignof(GTimeVal));
    printf("%s;%zu;%zu\n", "GTokenType", sizeof(GTokenType), alignof(GTokenType));
    printf("%s;%zu;%zu\n", "GTokenValue", sizeof(GTokenValue), alignof(GTokenValue));
    printf("%s;%zu;%zu\n", "GTrashStack", sizeof(GTrashStack), alignof(GTrashStack));
    printf("%s;%zu;%zu\n", "GTraverseFlags", sizeof(GTraverseFlags), alignof(GTraverseFlags));
    printf("%s;%zu;%zu\n", "GTraverseType", sizeof(GTraverseType), alignof(GTraverseType));
    printf("%s;%zu;%zu\n", "GTuples", sizeof(GTuples), alignof(GTuples));
    printf("%s;%zu;%zu\n", "GUnicodeBreakType", sizeof(GUnicodeBreakType), alignof(GUnicodeBreakType));
    printf("%s;%zu;%zu\n", "GUnicodeScript", sizeof(GUnicodeScript), alignof(GUnicodeScript));
    printf("%s;%zu;%zu\n", "GUnicodeType", sizeof(GUnicodeType), alignof(GUnicodeType));
    printf("%s;%zu;%zu\n", "GUnixPipe", sizeof(GUnixPipe), alignof(GUnixPipe));
    printf("%s;%zu;%zu\n", "GUnixPipeEnd", sizeof(GUnixPipeEnd), alignof(GUnixPipeEnd));
    printf("%s;%zu;%zu\n", "GUriError", sizeof(GUriError), alignof(GUriError));
    printf("%s;%zu;%zu\n", "GUriFlags", sizeof(GUriFlags), alignof(GUriFlags));
    printf("%s;%zu;%zu\n", "GUriHideFlags", sizeof(GUriHideFlags), alignof(GUriHideFlags));
    printf("%s;%zu;%zu\n", "GUriParamsFlags", sizeof(GUriParamsFlags), alignof(GUriParamsFlags));
    printf("%s;%zu;%zu\n", "GUriParamsIter", sizeof(GUriParamsIter), alignof(GUriParamsIter));
    printf("%s;%zu;%zu\n", "GUserDirectory", sizeof(GUserDirectory), alignof(GUserDirectory));
    printf("%s;%zu;%zu\n", "GVariantBuilder", sizeof(GVariantBuilder), alignof(GVariantBuilder));
    printf("%s;%zu;%zu\n", "GVariantClass", sizeof(GVariantClass), alignof(GVariantClass));
    printf("%s;%zu;%zu\n", "GVariantDict", sizeof(GVariantDict), alignof(GVariantDict));
    printf("%s;%zu;%zu\n", "GVariantIter", sizeof(GVariantIter), alignof(GVariantIter));
    printf("%s;%zu;%zu\n", "GVariantParseError", sizeof(GVariantParseError), alignof(GVariantParseError));
    printf("%s;%zu;%zu\n", "GWin32OSType", sizeof(GWin32OSType), alignof(GWin32OSType));
    return 0;
}
